#! python

# #####################################################################
# Copyright (c) 2017, Stanley Engle
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# #####################################################################

# Standard library imports.
import argparse
import sys


# The version number of this script/module.
__version__ = '2'


def calculate(air_temp_c, wind_speed_kph, on_error = float('nan')):
    """Calculate the wind chill.

    The wind chill is calculated using the air temperature in degrees 
    Celsius and the wind speed in kilometers per hour.  See:
    http://www.nws.noaa.gov/om/winter/windchill.shtml

    Parameters
    ----------
    air_temp_c : float
        The air temperature in degrees Cesius.
    wind_speed_kph : float
        The wind speed in kilometers per hour.
    on_error : optional, float('nan')
        The return value when an error occcurs during calculation.  May 
        be any type, but preferably a float.

    Returns
    -------
    float
        The wind chill in degrees C.

    """
    try:

        at_c = float(air_temp_c)
        ws_kph = float(wind_speed_kph)

        wc = 13.12 + (0.6215 * at_c) - \
            (11.37 * (ws_kph ** 0.16)) + \
            (0.3965 * at_c * (ws_kph ** 0.16))

        if (at_c >= 10.0) or (ws_kph <= 4.828):
            wind_chill = at_c

        else:
            wind_chill = wc

    except:
        wind_chill = on_error

    return wind_chill


def run(air_temp_c, wind_speed_kph, on_error = float('nan')):
    """Calls the calculate function.  See calculate docstring.

    """
    return calculate(air_temp_c, wind_speed_kph, on_error = on_error)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('air_temp_c', 
        type = float, 
        help = 'The air temperature in degrees Celsius.')
    parser.add_argument('wind_speed_kph', 
        type = float, 
        help = 'The wind speed in kilometers per hour.')

    # Parse the command line arguments
    args = parser.parse_args()

    # Call the run function.
    wind_chill = run(args.air_temp_c, args.wind_speed_kph)

    # Print the output to standard out.
    sys.stdout.write(str(wind_chill) + '\n')

    # Explicitly exit the script.
    sys.exit(0)


